@@ -15,6 +15,8 @@ c
          call epDraw_cap(comp, p, n)
       elseif(comp.struc(1:8) .eq. 'ellips') then
          call epDraw_ellips(comp, p, n)
+      elseif(comp.struc(1:8) .eq. 'angle') then
+         call epDraw_angle(comp, p, n)
 c---  elseif(comp.struc(1:8) .eq. '%2') then
 c---     call epDraw_%2(comp, p, n)
       else
