@@ -158,5 +158,24 @@ c        at present, we don't use above two factors
 c 
       cf = 1./(1. + c1 * dedx)
       end
+      subroutine epGetQuenchCf(dedx, Bircs, cf)
+      implicit none
+      real*8  dedx  ! input.  dE/dx (GeV/(g/cm^2) for the partcle
+      real*8  Bircs(3) !  input. Bircs correction factor
+      real*8  cf      ! output. correction factor. dE_eff = cf x dE_true
 
+c      if( abs( aPtcl.charge ) .gt. 1)  then
+c        c1 = c1 * media.BirksCC
+c      endif
+c      cf = 1./(1. + c1 * dedx + media.BirksC2 * dedx**2)
+c        at present, we don't use above two factors
+c 
+      real*8 cc,c1, c2
+      c1 = Bircs(2)
+c      if( Bircs(1) .gt. 0.  .and. Bircs(1) .ne. 1.) then
+c         c1 = c1 * Bircs(1)      
+c      endif
+c     cf = 1./(1. + c1 * dedx + Birks(3) * dedx**2)
+      cf = 1./(1. + c1 * dedx)
+      end
 
