@@ -10,14 +10,23 @@ c
       subroutine epgen
       implicit none
 #include  "ZepTrackv.h"
+#include  "ZepTrackp.h"
 #include  "Zcnfig.h"
+#include  "ZsepManager.h"
+c >>>>>>>>>>>>>>>light
+      integer,save::onlyonce
+c      common /dddebug/ onlyonce
+c<<<<<<<<<<<<<
       integer icon
-
 c         init. for 1 event has been finished, next is
 c       to put other final init. for 1 event
+
       call epr1ev
+c>>>>>>>>>>>>>>>>>>>light
+      onlyonce = 0
+c<<<<<<<<<<<<<<<<<<
 c         *** until loop*** 
-      icon =0
+      icon = 0
       do while (icon .eq. 0)
 c                get 1 particle from stack area 
          call eppop(cTrack, icon)
@@ -25,8 +34,33 @@ c                icon=0: 1 ptcl gotten in cTrack
 c                       -1: no more ptcl  in the stack
          if(icon .eq. 0) then
             call epfl1
+         else
+           !  all of normal tracking finished.>>>>>>>>>>>>>>>>light
+            if( Light == 12 .and. onlyonce == 0 ) then
+              ! energy deposit has been stored in cells.  we must generate
+              ! scinti light and do ray tracing.  We first push Edepo as
+              ! psudo ptcls
+               call epLightPushCells(0, 0)
+               icon = 0  ! now many cells stacked; reset icon
+               onlyonce = 1
+            endif   
+           !<<<<<<<<<<<<<<<<<<<<<<<<<<
          endif
       enddo
+       !>>>>>>>>>>>>>>>>>>>>>>>light
+      if( Light == 21 ) then
+         ! cell stored deposit should be put in a disk  as primary
+         ! for another job (file name with + or -)
+         if( FirstC ) then
+            ! no interatcion of incident so far; we must write
+            ! event header
+            call epLightIOwrite1stCol
+         endif
+         call epLightIOwriteCell
+         call epLightIOwritedE(OutPrimEff, Det.nct)
+             ! enery deposit of all comp. with d /=0
+      endif
+       !<<<<<<<<<<<<<<<<<<<<<<
       end
       subroutine epicosmos(param)
       implicit none
@@ -46,9 +80,9 @@ c          read parameter file for Cosmos.
       implicit none
 c             init for all event; read epics data and cofig data
 #include  "ZepManager.h"
+#include  "ZsepManager.h"
 #include  "ZepTrackv.h"
 #include  "ZepTrackp.h"
-#include  "ZepStack.h"
 #include  "Zcnfig.h"
 
 
@@ -57,13 +91,14 @@ c             init for all event; read epics data and cofig data
 
       integer klena
       character*8 uid
-
+      integer i, icon
 c
 c                 open basic data residence file for epics
 c              component # is undefined yet
       Cn=-1
-      Stack_pos = 0
-      StackDisk = 0
+      call epempty
+c      Stack_pos = 0
+c      StackDisk = 0
       Nevrun=0
 c                read basic parameter
       call epprmr(dsn1)
@@ -82,6 +117,20 @@ cc       call chanwnC
 c       ################
 c                user dependent init
        call uiaev
+c>>>>>>>>>>>>>>>>light
+c         if Light >0, manipulate CountDE of each component
+       if(Light >  0) then
+          call epLightCountDE ! 0 means from system
+
+          call epLightAlloc  ! alloc arrays
+          if(Light == 21) then
+             call epLightIOwriteIni
+          endif
+       endif
+c<<<<<<<<<<<<<<
+       !  allocate Eloss array for all comp. (always)
+       call epAlloc(Det.nct)
+
        if(MagField .eq. 1)then
           Bfield.x = Bxu
           Bfield.y = Byu
@@ -120,29 +169,34 @@ c             init for 1 event
        Firsti=.true.
        FirstC=.true.
        Proc1 = '   '   ! first collision process.
-       FirstInt.x = -100000
-       FirstInt.y = -100000
-       FirstInt.z = -100000
-       
+       FirstInt.x = -100000.d0
+       FirstInt.y = -100000.d0
+       FirstInt.z = -100000.d0
+       Nevrun = Nevrun + 1
        pathInB=0.
        SumDe = 0.
        Move.Abort = 0     !  if this become non zero for an event, 
                      !  the particle is discarded or event is discarded
 c                 user dependent init
+c>>>>>>>>>>>>>>>Light
+       if(Light > 0) then
+          call epLighti1ev
+          if(Light == 21 ) then
+             call epLightIOwriteiev
+          endif
+       endif
+c<<<<<<<<<<<<<<<<<
        call ui1ev
-
-
 c               next call sets to icon=1,    if usetip is not
 c               user made.
        call usetip(icon)
 
 
-
 c                 icon=0-->user routine already set prtcl
 c                 icon=1-->no user routine for incident
        if(Trace) then
           write(filen, *) TraceDir(1:klena(TraceDir))//'/trace',
-     *         Nevrun + 1
+     *         Nevrun 
           call kseblk(filen, ' ', leng)
           call copenfw(abs(IoTrace), filen,  jcon)
           if(jcon .ne. 0) then
@@ -167,8 +221,13 @@ c      ******************* end of 1 event
 #include  "ZepTrackp.h"
 #include "ZepTrackv.h"
 
-       Nevrun = Nevrun + 1
+c>>>>>>>>>>>>>>>>>Light
+       if(Light >  0) then
+          call epLighte1ev
+       endif
+c<<<<<<<<<<<<<<<<<<<
 c               user dependent end process
+
        if(Move.Abort .le. 1) call ue1ev
        if(Trace) then
           close(abs(IoTrace))
@@ -178,13 +237,19 @@ c      *************  now 1 event is ready to start
        subroutine epr1ev
        implicit none
 #include "ZepTrackv.h"
+#include "ZepTrackp.h"
 
        integer icon
 
 c          the procedures to be performed after the completion of
 c          init. for 1 event must be placed here
 c          save first stacked track as  incident
-        call epgetTrack(1, Incident, icon)
+       if( Light /= 22 ) then
+               ! in the case of 22, primary info has been
+               ! read alredy, since it is placed at the top
+               ! of each event
+          call epgetTrack(1, Incident, icon)
+       endif
 c        if(icon .ne. 0) then
 c           call cerrorMsg('no incident found in epr1ev', 0)
 c        endif
@@ -198,32 +263,6 @@ c                user dependent all event end
 
        call ueaev
        end
-c      ****************************
-      subroutine epqstn(n)
-      implicit none
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
-
-       integer n
-c               inquire current stack #
-       n = Stack_pos
-       end
-c      *************************
-       subroutine epqstt(n, aTrack)
-       implicit none
-#include  "ZepTrack.h"
-#include  "ZepStack.h"
-c        inquire a stacked particle in the n-th stack position
-       integer  n  !  input. stackposition
-       record /epTrack/  aTrack  !  output.  stacked track info.
-                                 !  if n is invalid, aTrack.p.fm.p(4)=0
-
-       if(n .ge. 1 .and. n .le. Stack_pos) then
-          aTrack =  Stack(n)
-       else
-          aTrack.p.fm.p(4) = 0. 
-       endif
-       end
 c      ***********************
        subroutine epqevn(nev)
       implicit none
@@ -249,63 +288,76 @@ c              until current partile dies or all are put into stack.
 c                current track is in  cTrack
 c          fix process and free path
 c          and compute tentative new pos.--> Move
-
          call epnewp(icon)
+
          if(icon .ne. 0) goto 100 ! abort is specified by the user
 c          energy loss count in user hook
-         if(Det.cmp(Cn).CountDE .gt. 0 .and.
-     *           cTrack.p.charge .ne. 0) then
-            if( Move.Track.p.fm.p(4)-Move.Track.p.mass
+c         if(Det.cmp(Cn).CountDE .gt. 0 .and. >>>>>>>>>light
+          if(  cTrack.p.charge .ne. 0) then  ! <<<<<<<<<
+             if( Move.Track.p.fm.p(4)-Move.Track.p.mass
      *                                     .le. KEmin ) then 
-               info = 1
-            else
-               info = 0
-            endif
-            call userde(info, cTrack, Move, Media(MediaNo))
-         endif
+                info = 1
+             else
+                info = 0
+             endif
+
+             call epLightPreUserde(info, cTrack)
+          endif
 c          add  time  
-         if(TimeStruc) then
-            call epaddTime
-         endif
+          if(TimeStruc) then
+             call epaddTime
+          endif
 
 c          energy check
-         call epchckE(Move.Track, icon)
+          call epchckE(Move.Track, icon)
 
 
-
-         if(icon .eq. 0 ) then
+          if(icon .eq. 0 ) then
 c           adjust momentum; because of  energy change
-            if(cTrack.p.charge .ne. 0) then
-               call epe2p(Move.Track.p)
-            endif
+             if(cTrack.p.charge .ne. 0) then
+                call epe2p(Move.Track.p)
+             endif
 
 c            if(.not. Move.Cross .and. cTrack.p.charge .ne. 0) then
 c               we don't use above judgement; later at epCross, we use new
 c               angle due to scattering at Move.boundary. (don't use
 c               new  position due to scattering in epCross)
 c           ++++++++++
-            if(cTrack.p.charge .ne. 0) then
+             if(cTrack.p.charge .ne. 0) then
 c                 multiple scattering, magnetic deflection
 c                 electric deflection
-               call epdeflection(icon)
+                call epdeflection(icon)
 
-            endif
+             endif
 c              take trace info.
-            if(Trace) then
-               if(IoTrace .lt. 0 .or. (IoTrace .gt. 0 
-     *           .and. cTrack.p.charge .ne. 0)) then
-                  call epTrace
-               endif
-            endif
+             if(Trace) then
+                if(IoTrace .lt. 0 .or. (IoTrace .gt. 0 
+     *               .and. cTrack.p.charge .ne. 0)) then
+                   call epTrace
+                endif
+             endif
 c              really move
 #ifdef  SUBSTREC
             cTrack = Move.Track
 #else
             call epsubstTRK( cTrack, Move.Track)
 #endif
+
             if(.not. Move.Trunc) then
+
                 call epint(icon)       ! interaction routine
 c                       icon =  1 always.
+#ifdef LightINTINFO
+                if(Move.Track.p.code == klight ) then
+                   call userintinfo
+                endif
+#endif
+#ifdef OtherINTINFO
+                if(Move.Track.p.code > 0) then
+                   call userintinfo
+                endif
+#endif
+
             elseif(Move.Cross) then
                call epCross(icon)
             endif
@@ -361,10 +413,19 @@ c       do for each ptcl
          call epprog  ! Move.proc, Move.dt fixed
       elseif(cTrack.p.code .eq. kelec) then
          call epproe  ! Move.proc, Move.dt fixed
-      else
+      elseif( cTrack.p.code > 0 ) then
          call epNonEleMag
+c>>>>>>>>>>>>>light
+      elseif( Light > 0 ) then
+         if( cTrack.p.code == klight ) then
+            call epproLight
+         else
+            write(0,*) " invalid  code=",cTrack.p.code
+            write(0,*) " for light related ptcl's path sampling"
+            stop
+         endif
       endif
-      
+c<<<<<<<<<<<<<<<<<<<
       if(Firsti) then
          Firsti=.false.
          if(.not. FreeC) then
@@ -386,6 +447,7 @@ c              local. coord..
 
 c     *************** see if Move.Track crosses the boundary ******
 c              set Cross=t/f, if t, adjust path and set Trunc=t
+
       call epifCross(icon)
                                                                            
       if(icon .eq. 2) return  !  ptcl went void by delicate boundary.
@@ -395,7 +457,8 @@ c
 c     ************** energy loss consideration ******************
 c        if E becomes <= m,  adjust path and set Trunc=t,
 c        and reset Cross.
-
+c            kchgPath case Move.dl=0 so light related
+c            ptcl will not goto epEloss
          if(cTrack.p.charge .ne. 0 .and. Move.dl .gt. 0.) then
             call epEloss
          else
@@ -410,6 +473,8 @@ c          manager when a particle crosses the boudarynon
 #include "ZepTrackv.h"
 #include "ZepTrackp.h"
 #include "Zcnfig.h"
+#include "Zcode.h"
+
       integer icon ! input/ouptut  1--> discard this particle
 c                                 other values are not given here
       integer cnx, info
@@ -426,44 +491,44 @@ c       cross the boundary;  use  new angle at the boundary.
 c      (not cTrack.w.x etc)
       cTrack.pos.x = Move.boundary.x +
      *     EpsLeng* Move.Track.w.x  
-c          if EpsLeng and w.x are both small, and x is large
-c          no move may happen, avoid such case.
-c               You may give larger EpsLeng in input data.
-c
-      if( cTrack.w.x .ne. 0. ) then
-         n = 2
-         do while( cTrack.pos.x .eq. Move.boundary.x .and.
-     *             n .lt. 10 ) 
-            cTrack.pos.x = Move.boundary.x +
-     *           n*EpsLeng* Move.Track.w.x
-            n = n * 2
-         enddo
-      endif
-
+ccccc          if EpsLeng and w.x are both small, and x is large
+ccccc          no move may happen, avoid such case.
+ccccc               You may give larger EpsLeng in input data.
+ccccc
+ccc      if( cTrack.w.x .ne. 0. ) then
+ccc         n = 2
+ccc         do while( cTrack.pos.x .eq. Move.boundary.x .and.
+ccc     *             n .lt. 10 ) 
+ccc           cTrack.pos.x = Move.boundary.x +
+ccc     *           n*EpsLeng* Move.Track.w.x
+ccc            n = n * 2
+ccc         enddo
+ccc      endif
+ccccccc
       cTrack.pos.y = Move.boundary.y +
      *     EpsLeng* Move.Track.w.y
-      if( cTrack.w.y .ne. 0.) then
-         n = 2
-         do while( cTrack.pos.y .eq. Move.boundary.y
-     *             .and. n .lt. 10 ) 
-            cTrack.pos.y = Move.boundary.y +
-     *           n*EpsLeng*  Move.Track.w.y
-            n = n * 2
-         enddo
-      endif
-
+ccc      if( cTrack.w.y .ne. 0.) then
+ccc         n = 2
+ccc         do while( cTrack.pos.y .eq. Move.boundary.y
+ccc     *             .and. n .lt. 10 ) 
+ccc           cTrack.pos.y = Move.boundary.y +
+ccc     *           n*EpsLeng*  Move.Track.w.y
+ccc           n = n * 2
+ccc         enddo
+ccc      endif
+ccc
       cTrack.pos.z = Move.boundary.z +
      *     EpsLeng* Move.Track.w.z
-      if(cTrack.w.z .ne. 0. ) then
-         n = 2
-         do while( cTrack.pos.z .eq. Move.boundary.z  .and.
-     *          n .lt. 10 )
-            cTrack.pos.z = Move.boundary.z +
-     *           n*EpsLeng* Move.Track.w.z
-            n = n * 2
-         enddo
-      endif
-
+ccc       if(cTrack.w.z .ne. 0. ) then
+ccc         n = 2
+ccc         do while( cTrack.pos.z .eq. Move.boundary.z  .and.
+ccc      *          n .lt. 10 )
+ccc            cTrack.pos.z = Move.boundary.z +
+ccc      *           n*EpsLeng* Move.Track.w.z
+ccc            n = n * 2
+ccc          enddo
+ccc       endif
+ccc 
 #ifdef  SUBSTREC
       cTrack.w = Move.Track.w
 #else
@@ -473,7 +538,9 @@ c
 c         new comp. number;  new position would be, in principle,
 c         in a new component. but scattering at the boundary may
 c         cause the new pos to be in the same component.
-      call eppos2cn(Cn, cTrack, cnx)
+      call eppos2cn(Cn, cTrack, cnx)   ! cnx is not yet set in cTrack
+ 
+
       if(Cn .eq. cnx) then
          Move.Cross = .false.
       elseif(cnx .gt. Det.nct) then
@@ -482,9 +549,38 @@ c         cause the new pos to be in the same component.
          info = -cnx   ! exiting to cnx
       endif
       cTrack.cn = cnx
+c         update coord. to local one in new  comp.; this part was inside
+c              " elseif(Move.Cross) then"
+c         which appears later; since it is used in LightAbBndry
+      if(Move.Cross) then
+         call epl2w(Cn, cTrack.pos, postemp)
+         call epw2l(cnx, postemp,  cTrack.pos)
+         call epl2wd(Cn, cTrack.w,  dirtemp)
+ccc         call epw2ld(cnx, dirtemp, cTrack.w)
+         call epw2ldm(cnx, dirtemp, cTrack.w, cTrack.p)
+      endif
+
+       !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>light
+      if(Move.Cross .and.  cTrack.p.code < 0 ) then
+         call epLightAtBndry( cnx, icon )
+            ! cTrack, Move and Cn are implicit  in/out param.
+            ! if refraction happens, cTrack's direction becomes 
+            !    refracted direc. pos, cnx are unchaged
+            !    icon = 0
+            ! if reflection happens, Move.Cross=F, cnx will become Cn.
+            !    cTrack's pos  is made to be that of  Move. 
+            !    icon = 0
+            ! if absorbed at the boundary,  icon = 1 Move.Cross=F
+            ! if light passes thru the component, icon = 0. cTrack
+            !   unchaged. cnx unchaged
+      endif
+         !<<<<<<<<<<<<<<<<<<<<<<<<<<<
       if(Det.cmp(Cn).CountIO .ge. 2 .and. Move.Cross) then
 c          user hook for  counting exiting ptcls 
 c                    exiting from Cn to cnx
+         if(Light > 0 .and. aTrack.p.code == klight) then
+            call epLightPC(info)  ! photon counter  for exiting light
+         endif
          call userbd(info, aTrack, Move, Media(MediaNo))
          if(Move.Abort .ne. 0) then
             if(Move.Abort .eq. 3) then
@@ -495,24 +591,15 @@ c                    exiting from Cn to cnx
             endif
          endif
       endif
-c ///////////
-c      call debugpos('epCross; eppos2cn', Cn, cTrack.pos, cnx)
-c////////////////
+
 
       if(cnx .gt. Det.nct) then
 c          void
          icon = 1
       elseif(Move.Cross) then
          info = Cn            ! save the current cn
-c         update coord. to local one in new  comp.
-         call epl2w(Cn, cTrack.pos, postemp)
-         call epw2l(cnx, postemp,  cTrack.pos)
-         call epl2wd(Cn, cTrack.w,  dirtemp)
-ccc         call epw2ld(cnx, dirtemp, cTrack.w)
-         call epw2ldm(cnx, dirtemp, cTrack.w, cTrack.p)
-         
 c            update comp. info (Cn, MediaNo etc are updated)
-         call epnewComp  
+         call epnewComp(cTrack)
 #ifdef  SUBSTREC
          aTrack = Move.Track
 #else
@@ -527,6 +614,11 @@ c            update comp. info (Cn, MediaNo etc are updated)
          if(Det.cmp(Cn).CountIO .eq. 1 .or.
      *      Det.cmp(Cn).CountIO .eq. 3 ) then
 c             count entering ptcls;  from info to Move.Track.cn (=Cn)
+            if( Det.cmp(Cn).CountIO == 1 ) then
+               if(Light > 0  .and. aTrack.p.code == klight) then
+                  call epLightPC(info) ! photon counter  for entering light
+               endif
+            endif
             call userbd(info, aTrack, Move, Media(MediaNo)) 
             if(Move.Abort .ne. 0) then
                if(Move.Abort .eq. 3) then
@@ -559,7 +651,10 @@ c     **********************
 #include "ZepTrackp.h"
 #include "ZepTrackv.h"
 #include "Zcode.h"
-
+c////////////////                                                              
+c      logical show
+c      common /showshow/ show
+c///////////       
 
 c          sample interaction length for non-e/g
       real*8 prob, path, tokgpm2
@@ -573,6 +668,21 @@ c      call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
       call cfixModel( cTrack.p )
       call ciniSmpIntL
 c        call csampNEPIntL  before v7.0
+c/////////////
+c      if(cTrack.p.fm.p(4) < 0.15 .and.
+c     *   cTrack.p.code >2 ) then
+c        if( cTrack.p.code == 4 .and. cTrack.p.charge  == 0 ) then
+c           show = .false.
+c        else
+c           show = .true.
+c           write(0,*) ' ---> code=',cTrack.p.code
+c           write(0,*) 'E =', cTrack.p.fm.p(4)
+c        endif
+c      else
+c         show = .false.
+c      endif
+c/////////////  
+
       call epsmpNEPIntL(Media(MediaNo))
 c         reset muon interaction conditions
       call ep2cosCondr
@@ -633,7 +743,9 @@ c     ******************
      *      s1, s2, scol2, schg2, sigma
       real*8 cupsilon, csyncTELoss, cf
       logical lowehvyion
-
+       !>>>>>>>>>>>>>>>light
+      integer::epLightGetd  !  function for extracting d part from countDE
+       !<<<<<<<<<<<<<<<<<
       k = cTrack.p.code
 c              compute energy loss rate
       if(EdepdEdx) then
@@ -669,7 +781,6 @@ c                      heavy ion and ke/n is < 0.7 GeV
                   dedxmu = 0.
                endif
             endif
-
             if(Media(MediaNo).BirksC1 .gt. 0.) then
                if( abs(cTrack.p.charge) .gt. 1) then
 c                   quenching effect
@@ -685,7 +796,7 @@ c                   quenching effect
      *         Media(MediaNo).rhoc
             dedlmu = dedxmu /Media(MediaNo).gtocm * ! GeV/cm
      *         Media(MediaNo).rhoc
-            if( abs( Det.cmp(Cn).CountDE) .eq. 2) then
+            if( abs( epLightGetd( Det.cmp(Cn).CountDE) ) .eq. 2) then
 c                Enery loss fluctuation; use  Urban model for singl charge
 c                 or  high energy
                if( lowehvyion ) then
@@ -1023,13 +1134,27 @@ c          Move.dt,dx,dl is adjusted if truncated, and Trunc is set to T.
 c
 #include  "ZepTrackv.h"
 #include  "ZepTrackp.h"
+#include  "Zcode.h"
 
 c           max length movable
       real*8 tmax, r
-
       if(cTrack.p.charge .eq. 0) then
 c           neutral particle can move any length basically
          Move.Trunc = .false.
+c           but we set max r.l be 100 r.l so that
+c           boundary calculation has high accuracy
+         if(Move.dt .gt. 100.) then
+            Move.dt = 100.
+            Move.dx = Move.dt * Media(MediaNo).X0g
+            Move.dl = Move.dx * Media(MediaNo).gtocm /
+     *           Media(MediaNo).rhoc
+            Move.Trunc = .true.
+         endif            
+      elseif( Light > 0  .and.  cTrack.p.code == kchgPath ) then
+         Move.Trunc = .false.   
+             ! Move.dt  etc is 0
+
+c<<<<<<<<<<<<<<<<<
 c      elseif(Move.proc .eq. 'no') then
 c         Move.Trunc=.false.
 c         if(MagField .gt. 0 .or.  ElecField .gt. 0) then
@@ -1162,9 +1287,27 @@ c
       logical ok, kbtest
       integer k
       real*8 ke
+c////////////////                                                              
+c      logical show
+c      common /showshow/ show
+c///////////       
 
       ke = aTrack.p.fm.p(4)- aTrack.p.mass
       k =  aTrack.p.code
+c >>>>>>>>>>>>>>>>>>light
+      if(k .eq. klight) then
+         if(Light <=  0 ) then
+            write(0,*) ' Light =0 but light appeared'
+            icon = 1   ! light is not treated, so death
+            stop
+         endif
+         call epLightchkE( aTrack, icon) ! check wave length
+         return  ! *************
+      endif
+c<<<<<<<<<<<<<<<<<<<<light
+      if(k == kgnuc ) then
+         ke = ke/aTrack.p.subcode
+      endif
 
       if(k .eq. kelec) then
           if(aTrack.p.charge .eq. -1) then
@@ -1175,9 +1318,9 @@ c
                    Move.dEeff = ke
                    Move.dEioni= Move.dE  
                    SumDe = SumDe + Move.dE
-                   if(Det.cmp(Cn).CountDE .ge. 1) then
-                      call userde(1, aTrack, Move, Media(MediaNo))
-                   endif
+c                   if(Det.cmp(Cn).CountDE .ge. 1) then >>>>>>light
+                      call epLightPreUserde(1, aTrack)
+c                   endif                               <<<<<<<<<<
                 endif
              endif
           else
@@ -1188,13 +1331,11 @@ c              positron
                    Move.dE = aTrack.p.fm.p(4) + masele
 
                    SumDe = SumDe + Move.dE
-c /////            Move.dEeff = ke
                    Move.dEeff = Move.dE
-c/////
                    Move.dEioni= Move.dE  
-                   if(Det.cmp(Cn).CountDE .ge. 1) then
-                      call userde(1, aTrack, Move, Media(MediaNo))
-                   endif
+c                   if(Det.cmp(Cn).CountDE .ge. 1) then >>>>>>>light
+                      call epLightPreUserde(1, aTrack)
+c                   endif                               <<<<<<<<<<< 
 
                 endif
              else
@@ -1213,14 +1354,15 @@ c                   if energy is very low, forced anihilation
                 Move.dEeff = ke
                 Move.dEioni= Move.dE  
                 SumDe = SumDe + Move.dE
-                if(Det.cmp(Cn).CountDE .ge. 1) then
-                   call userde(1, aTrack, Move, Media(MediaNo))
-                endif
+c                if(Det.cmp(Cn).CountDE .ge. 1) then  >>>>>>>>>>>light
+                   call epLightPreUserde(1, aTrack)
+c                endif                                <<<<<<<<<<<
              endif
           endif
        elseif(k .eq.  knuc) then
           if(aTrack.p.subcode  .eq. regptcl) then
-             ok= ke .gt. KEmin
+c             ok= ke .gt. KEmin
+             ok= ke .gt. EminH
              if(.not. ok) then
                 if( (aTrack.p.charge .eq. 1 .and. 
      *             kbtest(Eabsorb, BitProton)) .or.
@@ -1230,9 +1372,9 @@ c                   if energy is very low, forced anihilation
                    Move.dEeff = ke
                    Move.dEioni= Move.dE  
                    SumDe = SumDe + Move.dE
-                   if(Det.cmp(Cn).CountDE .ge. 1) then
-                      call userde(1, aTrack, Move, Media(MediaNo))
-                   endif
+c                   if(Det.cmp(Cn).CountDE .ge. 1) then >>>>>>>>>>>>light
+                      call epLightPreUserde(1, aTrack)
+c                   endif                               <<<<<<<<<<<  
                 endif
              endif
           else
@@ -1246,38 +1388,61 @@ c                (dependent on the ptcl type)
                    Move.dEeff = ke
                    Move.dEioni= Move.dE  
                    SumDe = SumDe + Move.dE
-                   if(Det.cmp(Cn).CountDE .ge. 1) then
-                      call userde(1, aTrack, Move, Media(MediaNo))
-                   endif
+c                   if(Det.cmp(Cn).CountDE .ge. 1) then >>>>>>>>>>>>light
+                      call epLightPreUserde(1, aTrack)
+c                   endif                               <<<<<<<<<<<<<
                 endif
              endif
           endif
        elseif( k .eq.  kpion .or.
      *         k .eq.  kkaon .or.
-     *         k .eq.  kmuon ) then
+     *         k == kmuon ) then
 c            can decay
-          ok = aTrack.p.fm.p(4) .gt. KEmin
+c          ok = aTrack.p.fm.p(4) .gt. KEmin
+          ok = aTrack.p.fm.p(4) > EminH
+          if( ok .and. ke <= EminH ) then
+             if( k == kmuon .and.  aTrack.p.charge == -1) then
+                ! can be abosrbed by capture so follow until death
+             elseif( .not. Move.trunc ) then
+                ! force to decay
+                Move.proc='decay'
+                call cresetIntInf   ! this rest is needed
+             endif
+c/////////////////
+c             if(show) then
+c                write(0,*) ' in E check ;KE=', ke
+c                write(0,*) 'proc =',Move.proc, ' Move.dl =',Move.dl,
+c     *                  ' trunc =',Move.trunc
+c             endif
+c////////////////
+          endif
           if(.not.ok) then
              if(kbtest(Eabsorb,BitDecay)) then
                 Move.dE = ke
                 Move.dEeff = ke
                 Move.dEioni= Move.dE  
                 SumDe = SumDe + Move.dE
-                if(Det.cmp(Cn).CountDE .ge. 1) then
-                   call userde(1, aTrack, Move, Media(MediaNo) )
-                endif
+c                if(Det.cmp(Cn).CountDE .ge. 1) then  >>>>>>>>>>>>light 
+                   call epLightPreUserde(1, aTrack)
+c                endif                                <<<<<<<<<<<<<<<
              endif
           endif
        else
-          ok = ke .gt. KEmin
-          if(.not.ok) then
-             if(kbtest(Eabsorb, BitOther)) then
-                Move.dE = ke
-                Move.dEeff = ke
-                Move.dEioni= Move.dE  
-                SumDe = SumDe + Move.dE
-                if(Det.cmp(Cn).CountDE .ge. 1) then
-                   call userde(1, aTrack, Move, Media(MediaNo))
+          if(k == kgnuc ) then
+             ok = ke  > EminH
+          else
+             ok = ke  >  KEmin
+          endif
+          if(.not.ok) then 
+             if(k /= kneue .and. k/= kneumu ) then
+                if(kbtest(Eabsorb, BitOther)) then
+                   Move.dE = ke
+                   Move.dEeff = ke
+                   Move.dEioni= Move.dE  
+                   SumDe = SumDe + Move.dE
+c                if(Det.cmp(Cn).CountDE .ge. 1) then >>>>>>>>>>>>light
+                   call epLightPreUserde(1, aTrack)
+c                endif                               <<<<<<<<<<<<
                 endif
              endif
           endif
@@ -1294,6 +1459,9 @@ c            can decay
        end
 c      *****************
        subroutine epaddTime
+!           >>>>>>>>>>>>>>>light
+       use modepLightPty
+!           <<<<<<<<<<<<<<<
        implicit none
 c        new position is assumed to be fixed.
 c        (however, before scattering, and energy check).
@@ -1306,7 +1474,13 @@ c        update time and take trace
        real*8 beta1, betaav
 c
        if(Move.Track.p.mass .eq. 0.) then
-          betaav=1.d0
+c   >>>>>>>>>>>>>>>>>>>light
+          if(Move.Track.p.code < 0 .and. cLcompNo > 0 ) then
+             betaav = 1./Lcomp( cLcompNo )%refracN
+          else
+c  <<<<<<<<<<<<<<<<<<<<<<<
+             betaav=1.d0
+          endif
        else
           call cgetBeta( Move.Track.p,  betaav)
           if(betaav .lt. 0.98) then
@@ -1351,6 +1525,10 @@ c      ************************
 
       record /epPos/temppos
 c
+c/////////////
+c      logical show
+c      common /showshow/show
+c///////////////
 
       k = cTrack.p.code
 c         almost dummy setting 
@@ -1371,7 +1549,8 @@ c
             call ep2cosPtcl( cTrack.p )
 c              for small basic cross section case.
             call epfixTarget2(ActiveMdl, Media(MediaNo))
-            call ep2cosCond2(Media(MediaNo).colA, Media(MediaNo).colZ)
+            call ep2cosCond2(Media(MediaNo).colA, Media(MediaNo).colZ,
+     *          Media(MediaNo).colXs)
             call cphotop        ! Cosmos function
             call eppushPtcl(cTrack)  ! use pos. info from this ptcl
          elseif(Move.proc .eq. 'mpair') then
@@ -1397,6 +1576,26 @@ c              for small basic cross section case.
      *         '('' process='',a4, '' for e is undef.'')') Move.proc
             call cerrorMsg(msg, 0)
          endif
+       !>>>>>>>>>>>>>>>>>>>>>>light
+      elseif( k == klight) then
+         if(Move.proc == "pe" ) then
+                  ! photo electron generation at sensor
+            call epLightAtSensor
+         elseif( Move.proc == "rayl" ) then
+                  ! Rayleigh scattering; use Xray region fomulat since
+                  ! (1+cos^2)dcos
+            call epcoher
+         elseif( Move.proc == "absorb" ) then
+                  ! absorbed. nothing to do;  not push any thing
+         elseif( Move.proc == "wls" ) then
+                   ! wave length shift
+            call epLightPreWLS
+         else
+            write(0,*) ' light interacion=', Move.proc
+            write(0,*) ' not defined '
+            stop
+         endif
+        !<<<<<<<<<<<<<<<<<<<<<<
       else
          if(Move.proc .eq. 'knoc') then
 
@@ -1414,7 +1613,13 @@ c              for small basic cross section case.
          else
             call ep2cosPtcl( cTrack.p )
             call epfixTarget(ActiveMdl, Media(MediaNo))
-            call ep2cosCond2(Media(MediaNo).colA, Media(MediaNo).colZ)
+            call ep2cosCond2(Media(MediaNo).colA, Media(MediaNo).colZ,
+     *           Media(MediaNo).colXs)
+c//////////////
+c            if(show) then
+c               write(0,*)  ' inteNEP;proc=' , Move.proc
+c            endif
+c////////////////
             call cinteNEP       ! cosmos eppp
             call eppushPtcl( cTrack )
          endif
@@ -1444,6 +1649,9 @@ c              for small basic cross section case.
             call epl2w(cTrack.cn, FirstInt, temppos)
             FirstInt = temppos
             FirstMedia = Media(MediaNo)
+            if(Light == 21 ) then
+               call epLightIOwrite1stCol
+            endif
          endif
       endif
 
@@ -1650,9 +1858,9 @@ c            energy absorbed by atom is Eabs = Eshell= Eg-(Ee-Me)
           Move.dEioni = Move.dE
           SumDe = SumDe + Move.dE
 c                  regard it as deposited in the media
-          if(Det.cmp(Cn).CountDE .ge. 1) then
-             call userde(1, cTrack, Move, Media(MediaNo))
-          endif
+c          if(Det.cmp(Cn).CountDE .ge. 1) then >>>>>>>>>>>>>>light
+             call epLightPreUserde(1, cTrack)
+c          endif                               <<<<<<<<<<<<<
           Exray = 0.
        else
 c             bit 1 is not on; characteristic x-ray emmission;
@@ -1666,9 +1874,6 @@ c                2)  For such p.e effect, vacancy of electron level is
 c                    filled by X-ray emission ; No Auger electron emmission
 c                    is considered. 1)+2) are good approximation.
          Exray = max( cTrack.p.fm.p(4) - (eout - masele), 0.d0)
-c//////////
-c         write(0,*) ' ----------Exray=',Exray, ' eout=',eout
-c////////////
        endif
 c         emitted electron
        elec1 = cTrack
@@ -1686,13 +1891,14 @@ c         emitted xray; assume isotropic
        if( Exray  .gt.  0.) then
           xray = cTrack
           call cmkptc(kphoton, 0, 0, xray.p)
-          call rndc(cost)
-          cost = 2.0*cost-1.0
-          call kcossn(cs,sn)
-          sint = sqrt(1.-cost**2)
-          xray.w.x = cs*sint
-          xray.w.y = sn*sint
-          xray.w.z = cost
+          call episoAngle( xray.w )
+c          call rndc(cost)
+c          cost = 2.0*cost-1.0
+c          call kcossn(cs,sn)
+c          sint = sqrt(1.-cost**2)
+c          xray.w.x = cs*sint
+c          xray.w.y = sn*sint
+c          xray.w.z = cost
           xray.p.fm.p(4) = Exray
           call eptransVect(cTrack.w, xray.w, xray.w)
           call epe2p(xray)
@@ -2138,8 +2344,61 @@ c      if(dddd) then
          write(*,*) ' icon=',icon
 c      endif
       end
+      subroutine episoAngle( dir ) 
+      implicit none
+#include "ZepDirec.h"      
+      record /epDirec/ dir
+      real*8  cost, cs, sn, sint
 
+      call rndc(cost)
+      cost = 2.0*cost-1.0
+      call kcossn(cs,sn)
+      sint = sqrt(1.-cost**2)
+      dir.x = cs*sint
+      dir.y = sn*sint
+      dir.z = cost
+      end
 
+      subroutine epResetCountIO(cmpNo, countio) 
+      implicit none
+#include "ZepTrackv.h"
+#include "Zcnfig.h"
+      integer,intent(in):: cmpNo !  comp. #                                                                      
+      integer,intent(in):: countio  ! countio to be set                                                           
+      if(cmpNo >= 1 .and. cmpNo <= Det.nct) then
+         Det.cmp(cmpNo).CountIO = countio
+      else
+         write(0,*) 'Warning from epResetCountIO:'
+         write(0,*) ' specified comp.# ', cmpNo, ' non exsistent'
+      endif
 
+      end
+c//////////
+      subroutine  epfordebug(msg)
+      implicit none
+#include "ZepTrackv.h"
 
+      character*(*) msg  ! callers name is usually better
+      character(len=20) struc
 
+         write(0,*) '--------------- ', msg
+         write(0,*) ' ctrackpos=',cTrack.pos.x, 
+     *      cTrack.pos.y, cTrack.pos.z, ' wx,y,z=',
+     *      cTrack.w.x, cTrack.w.y, cTrack.w.z
+         write(0,*) ' M-w.x =',Move.Track.w.x, 
+     *        Move.Track.w.y, Move.Track.w.z
+         write(0,*) ' Move.track.pos=',Move.Track.pos.x,
+     *     Move.Track.pos.y,  Move.Track.pos.z, ' dl=',
+     *     Move.dl,' E and mass=', Move.dE, Move.Track.p.mass,
+     *    ' trucn=',Move.Trunc
+         write(0,*)
+     *     ' code=', cTrack.p.code,  ' chg=',cTrack.p.charge,
+     *     ' E=',  cTrack.p.fm.p(4), ' wl =',cTrack.wl
+         write(0,*) ' cross=',Move.Cross, ' Trunc=',Move.Trunc
+         write(0,*) ' cTrack.cn=',cTrack.cn, ' Move.cn=',
+     *    Move.track.cn, 'Cn=',Cn
+         write(0,*) ' media #=', MediaNo, ' medai=', Media(MediaNo).name
+         call epqstruc(Cn, struc)
+         write(0,*) ' struc =', struc
+
+      end
